//
// SydB☮x: seccomp and landlock based application sandbox with support for namespaces
// src/syd-norm.rs: Print the normalized path name followed by a newline and exit.
//
// Copyright (c) 2024 Ali Polatel <alip@chesswob.org>
//
// SPDX-License-Identifier: GPL-3.0-or-later

use std::{io::Write, os::unix::ffi::OsStrExt, process::ExitCode};

fn main() -> ExitCode {
    let mut args = std::env::args();

    match args.nth(1).as_deref() {
        None => {
            println!("Usage: syd-norm path");
            println!("Print the normalized path name followed by a newline and exit.");
        }
        Some(value) => {
            let path = syd::fs::normalize(value);
            let path = path.as_os_str().as_bytes();
            // SAFETY: We panic if writing to standard output fails.
            #[allow(clippy::disallowed_methods)]
            std::io::stdout().write_all(path).expect("write to stdout");
            println!();
        }
    }

    ExitCode::SUCCESS
}
