# libsydtime: Rust-based C LD_PRELOAD library to replace vDSO time calls with syscalls

[![SydB☮x](https://git.sr.ht/~alip/syd/blob/main/data/syd.png)](https://todo.sr.ht/~alip/syd)
[![GNU](https://web.archive.org/web/20221222061733if_/https://dev.exherbo.org/~alip/images/gnu.png)](https://www.gnu.org/philosophy/philosophy.html)
[![Linux](https://chesswob.org/jja/tux.png)](https://www.kernel.org/category/about.html)
[![Exherbo](https://web.archive.org/web/20230518155203if_/https://dev.exherbo.org/~alip/images/zebrapig.png)](https://www.exherbolinux.org/docs/gettingstarted.html)

`libsydtime` is a
[LD_PRELOAD](https://en.wikipedia.org/wiki/Dynamic_linker#Systems_using_ELF)
library to replace [vDSO](https://en.wikipedia.org/wiki/VDSO) time calls
with syscalls. This library is meant as a complement to
[SydB☮x](https://sydbox.exherbolinux.org)'s
[trace/deny_tsc](https://git.sr.ht/~alip/syd#tracedeny_tsc) sandbox
command.

Note, this is known to work on programs built with the [musl
libc](https://musl.libc.org/), however it does not work on programs
built with recent versions of the [GNU
Libc](https://www.gnu.org/software/libc/) due to glibc's [rtld
profiling](https://sourceware.org/git/?p=glibc.git;a=blob;f=elf/rtld.c;h=4f494b792e3f376ba8f4635d01480264edcd65eb;hb=HEAD)
which makes direct use of the [timestamp counter](https://en.wikipedia.org/wiki/Time_Stamp_Counter).

## Prerequisites

- Rust (1.56.1 or later)

## Building and Installing from Source

To build and install `libsydtime` from source, follow these steps:

1. Clone the repository:
   ```bash
   git clone https://git.sr.ht/~alip/syd
   cd time
   ```
2. Use the Makefile which utilizes Cargo to build
   ```bash
   make
   make test
   ```
3. Switch to root and install
   ```
   sudo make install
   ```

## Usage

```
$ strace -qf -e%clock syd -ppaludis -mtrace/deny_tsc:1 -eLD_PRELOAD=/usr/local/lib/libsydtime.so date
[pid 27095] clock_gettime(CLOCK_REALTIME, {tv_sec=1706186439, tv_nsec=28829866}) = 0
Thu Jan 25 12:40:39 UTC 2024
[pid 27095] +++ exited with 0 +++
$ strace -qf -e%clock syd -ppaludis date # vDSO
Thu Jan 25 12:41:25 UTC 2024
[pid 27103] +++ exited with 0 +++
$ strace -qf -e%clock syd -ppaludis -mtrace/deny_tsc:1 date # vDSO not permitted
[pid 27113] --- SIGSEGV {si_signo=SIGSEGV, si_code=SI_KERNEL, si_addr=NULL} ---
[pid 27113] +++ killed by SIGSEGV +++
$
```

## BUGS

```
Hey you, out there beyond the wall,
Breaking bottles in the hall,
Can you help me?
```

Report bugs to ___SydB☮x___'s bug tracker at <https://todo.sr.ht/~alip/syd/>:
1. Always **be polite**, respectful, and kind:
   <https://css-tricks.com/open-source-etiquette-guidebook/>
2. Keep your final change as **small and neat** as possible:
   <https://tirania.org/blog/archive/2010/Dec-31.html>
3. Attaching poems with the bug report encourages consideration tremendously.

## COPYRIGHT

- **SPDX-License-Identifier:** *LGPL-3.0-or-later*
- **Copyright ©** 2023, 2024 Ali Polatel <alip@chesswob.org>

ChangeLog
=========

# 0.0.1

- Initial implementation of libsydtime
